"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createCICommand = (deps) => {
    let { formatter, linter, compiler, nyc, git, logger } = deps;
    return {
        execute: (options = {}) => __awaiter(this, void 0, void 0, function* () {
            let { tests = true } = options;
            let timestamp = new Date().getTime();
            let allTypescriptFiles = yield git.findAllTypescriptFiles();
            let results = yield Promise.all([
                compiler.runOnce([]),
                formatter.verifyFiles(allTypescriptFiles),
                linter.lintOnce(false, allTypescriptFiles),
                tests ? nyc.run() : Promise.resolve(true)
            ]);
            let toolErrors = results.filter(result => result === false).length;
            logger.log('ci', `CI tasks took ${Math.round((new Date().getTime() - timestamp) / 1000)} seconds`);
            if (toolErrors !== 0) {
                logger.error('ci', `${toolErrors} tool${toolErrors === 1 ? '' : 's'} reported errors`);
                return false;
            }
            return true;
        })
    };
};
//# sourceMappingURL=ci.js.map