"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createPrePushCommand = (deps) => {
    let { compiler, nyc, git, logger } = deps;
    return {
        execute: () => __awaiter(this, void 0, void 0, function* () {
            let timestamp = new Date().getTime();
            let pristine = yield git.isPristine();
            if (!pristine) {
                logger.error('pre-push', 'The working directory contains changes that are not committed.');
                logger.error('pre-push', 'The pre-push checks can therefore not be run');
                logger.error('pre-push', 'Please stash you work before pushing');
                return false;
            }
            let results = yield Promise.all([
                compiler.runOnce([]),
                nyc.run()
            ]);
            let toolErrors = results.filter(result => result === false).length;
            logger.log('pre-push', `Pre-push tasks took ${Math.round((new Date().getTime() - timestamp) / 1000)} seconds`);
            return toolErrors === 0;
        })
    };
};
//# sourceMappingURL=pre-push.js.map